package cn.kdan.cloud.pdf.office.common.utils;

import cn.kdan.cloud.pdf.office.common.enums.ValidStatusEnum;
import cn.kdan.cloud.pdf.office.common.vo.UserSubscriptionInfoVO;
import org.apache.commons.lang.RandomStringUtils;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 常用日期、自动生成id等方法工具类
 */
public class CommonUtils {

	public final static String DEFAULT_DATE_FORMAT = "yyyy-MM-dd HH:mm:ss";

	public final static String SHORT_DATE_FORMAT = "yyyy-MM-dd";

	public final static String NULL_STR = "";

	public final static String BLANK_STR = " ";

	public final static String UNDERSCORE_STR = "_";

	public final static String SLASH_STR = "/";

	public static final String ENCODING_ISO8859 = "ISO-8859-1";

	public static final String ENCODING_ASCII = "ascii";

	public static final String ENCODING_UTF8 = "utf-8";

	public static final String TEXT_HTML_UTF8 = "text/html; charset=utf-8";

	public static final String APPLICATION_JSON_UTF8 = "application/json; charset=utf-8";

	public static final int SECONDS_A_DAY = 24 * 60 * 60;

	public static final int MILLISECONDS_A_DAY = SECONDS_A_DAY * 1000;

	public static final String ENVIRONMENT = "Environment";

	public static final String IP_REXP = "([1-9]|[1-9]\\d|1\\d{2}|2[0-4]\\d|25[0-5])(\\.(\\d|[1-9]\\d|1\\d{2}|2[0-4]\\d|25[0-5])){3}";

	private static final int DEFAULT_ID_LENGTH = 9;
	/**
	 * 功能描述：获取当前日期时间格式化字符串，默认yyyy-MM-dd HH:mm:ss格式
	 */
	public static String getCurrentDateTime() {
		return getCurrentDateTime(DEFAULT_DATE_FORMAT);
	}

	/**
	 * 功能描述：获取当前日期时间格式化字符串
	 *
	 * @param dateFormat
	 *            日期时间格式
	 */
	public static String getCurrentDateTime(String dateFormat) {
		return new SimpleDateFormat(dateFormat).format(Calendar.getInstance()
				.getTime());
	}

	/**
	 * 功能描述：获取指定日期时间格式化字符串
	 *
	 * @param dateFormat
	 *            日期时间格式
	 */
	public static String getDateTime(String dateFormat, Date date) {
		return new SimpleDateFormat(dateFormat).format(date);
	}


	/**
	 *	 两个日期相差多少分钟
	 * @param endDate
	 * @param startDate
	 * @return
	 */
	public static long getDatePoor(Date endDate, Date startDate) {
		return (endDate.getTime() - startDate.getTime())/(1000 * 60);
	}
	/*
	 * 功能描述：生成对象的主键
	 *
	 * @return String
	 */
	public static String generateId() {
		UUID uuid = UUID.randomUUID();
		String id = uuid.toString();
//		id = id.replaceAll("-", "");
		return id;
	}    // 生成随机 8 位阿拉伯数字用户 ID
	public static String generateRandomUserId() {
		Random random = new Random();
		StringBuilder userId = new StringBuilder();

		for (int i = 0; i < 8; i++) {
			userId.append(random.nextInt(10)); // 生成 0-9 之间的随机数
		}

		return userId.toString();
	}

	/**
	 * 生成随机长度为9的字符串形式的id，使用apache common包中的api
	 * @return 生成的id
	 */
	public static String generateRandomStringId() {
		return generateRandomStringId(DEFAULT_ID_LENGTH);
	}

	/**
	 * 根据传入的参数长度 随机生成字符串
	 * @param count
	 * @return
	 */
	public static String generateRandomStringId(int count) {
		return RandomStringUtils.random(count, true, true);
	}

	/**
	 * 生成Openstack的资源名称
	 * @param prefix
	 * @return
	 */
	public static String generateName(String prefix){
		return prefix + "-" + Long.toHexString((new Date()).getTime());
	}


	/**
	 * 检查参数是否为空 或空串
	 * @param params
	 * @return
	 */
	public static boolean checkParams(String... params){
		if(params == null){
			return false;
		}
		for(String param : params){
			if(StringUtils.isEmpty(param)){
				return false;
			}
		}
		return true;
	}

	/**
	 * 自动生成带有当前日期的字符串
	 * @return
	 */
	public static String generateDateId() {
		Random random = new Random();
		int i = random.nextInt(9999) + 10000;
		String num = String.valueOf(i).substring(1);
		return getCurrentDateTime("yyyyMMddHHmmss") + num;
	}

	/**
	 * String格式的日期转为Date
	 * @param dateFormat 日期格式
	 * @param date
	 * @return
	 * @throws ParseException
	 */
	public static Date parseDate(String dateFormat, String date) throws ParseException {
		return new SimpleDateFormat(dateFormat).parse(date);
	}

	/**
	 * 逗号拼接list
	 * @param list
	 * @return
	 */
	public static String listToString(List<String> list){

		if(list==null){
			return null;
		}

		StringBuilder result = new StringBuilder();
		boolean first = true;

		//第一个前面不拼接","
		for(String string :list) {
			if(first) {
				first=false;
			}else{
				result.append(",");
			}
			result.append(string);
		}
		return result.toString();
	}

	/**
	 * 返回首字母大写后的字符串
	 *
	 * @param str
	 * @return
	 */
	public static String upperFirstLetter(String str){
		if(str.length() > 0){
			return org.apache.commons.lang.StringUtils.upperCase(str.substring(0, 1)) + str.substring(1, str.length());
		}
		return null;
	}



	/**
	 * 用*覆盖内容
	 * @param content
	 * @return
	 */
	private static String coverContent(String content){
		int length = content.length();
		int coverLength = Double.valueOf(Math.floor(length/2)).intValue();
		StringBuilder stringBuilder = new StringBuilder();
		if(coverLength == 0){
			return content;
		}else{
			int halfLength = Double.valueOf(Math.floor((length - coverLength)/2)).intValue();
			for (int i = 0;i < length;i++){
				if(i >= halfLength && (i < (halfLength + coverLength))){
					stringBuilder.append("*");
				}else{
					stringBuilder.append(content.charAt(i));
				}
			}
		}
		return stringBuilder.toString();
	}

	public static String generateVerifyCode() {
		return String.valueOf(new Random().nextInt(899999) + 100000);
	}

	public static String extractSubscriptionID(String orderID) {
		// 查找 ".." 出现的位置
		int dotIndex = orderID.indexOf("..");

		// 如果 ".." 存在，返回 ".." 前面的部分
		if (dotIndex != -1) {
			return orderID.substring(0, dotIndex);
		}

		// 如果没有 ".."，直接返回整个订单ID
		return orderID;
	}
	public static UserSubscriptionInfoVO findActiveSubscription(List<UserSubscriptionInfoVO> subscriptionInfoList, String model) {
		// 过滤掉isAi为true的记录
		List<UserSubscriptionInfoVO> filteredList = subscriptionInfoList.stream()
				.filter(info -> ObjectUtils.isEmpty(info.getIsAi()) || ValidStatusEnum.INVALID.value().equals(info.getIsAi())) // 排除AI记录
				.collect(Collectors.toList());
		UserSubscriptionInfoVO freeVersion = filteredList.stream()
				.filter(info -> info.getLevels().equals("1"))
				.findFirst()
				.orElse(null);

		// 找到一条生效的记录，按优先级排序
		Optional<UserSubscriptionInfoVO> activeSubscription = filteredList.stream()
				.filter(info -> !info.getLevels().equals("1"))
				.sorted(Comparator.comparingInt(UserSubscriptionInfoVO::getMaxDeviceNum).reversed()
						.thenComparing(Comparator.comparingInt(CommonUtils::getLevelPriority).reversed()) // 按优先级降序排列
						.thenComparing(Comparator.comparingInt(CommonUtils::getPlatforms).reversed())
						.thenComparing(Comparator.comparingInt(UserSubscriptionInfoVO::getPlatform).reversed())
						.thenComparing(Comparator.comparingInt(CommonUtils::getPlatformsCompare).reversed())// 如果优先级相同按max_device_num降序
				)
				.findFirst(); // 找到优先级最高的记录
		if (activeSubscription.isPresent() && activeSubscription.get().getStatus() == 2 && filteredList.size() > 2) {
			Optional<UserSubscriptionInfoVO> activeSubscription1 = filteredList.stream()
					.filter(user -> user.getStatus() == 1) // 过滤出 status = 1 的记录
					.sorted(Comparator.comparingInt(UserSubscriptionInfoVO::getMaxDeviceNum).reversed()
							.thenComparing(Comparator.comparingInt(CommonUtils::getLevelPriority).reversed()) // 按优先级降序排列
							.thenComparing(Comparator.comparingInt(CommonUtils::getPlatforms).reversed())
							.thenComparing(Comparator.comparingInt(UserSubscriptionInfoVO::getPlatform).reversed())
							.thenComparing(Comparator.comparingInt(CommonUtils::getPlatformsCompare).reversed())// 如果优先级相同按max_device_num降序
					)
					.findFirst();
			return activeSubscription1.orElse(freeVersion);
		}
		return activeSubscription.orElse(freeVersion); // 返回结果，如果没有找到则返回null
	}
	public static List<UserSubscriptionInfoVO> getProSubscription(List<UserSubscriptionInfoVO> subscriptionInfoList) {
		// 过滤掉isAi为true的记录
		List<UserSubscriptionInfoVO> filteredList = subscriptionInfoList.stream()
				.filter(info -> ObjectUtils.isEmpty(info.getIsAi()) || ValidStatusEnum.INVALID.value().equals(info.getIsAi())) // 排除AI记录
				.collect(Collectors.toList());
		return filteredList; // 返回结果，如果没有找到则返回null
	}

	public static UserSubscriptionInfoVO getProSubscriptionByModel(List<UserSubscriptionInfoVO> subscriptionInfoList,String model) {
		// 过滤掉isAi为true的记录
		UserSubscriptionInfoVO activeSubscription = subscriptionInfoList.stream()
				.filter(info -> ObjectUtils.isEmpty(info.getIsAi()) || ValidStatusEnum.INVALID.value().equals(info.getIsAi())) // 排除AI记录
				.filter(info -> info.getStatus() == 1) // user.getStatus() == 1
				.filter(info -> info.getPlatforms() != null && info.getPlatforms().toLowerCase().contains(model.toLowerCase())) // getPlatforms() 包含 model
				.sorted(Comparator.comparingInt(CommonUtils::getLevelPriority).reversed()// 如果优先级相同按max_device_num降序
				)
				.findFirst()
				.orElse(null);
		return activeSubscription; // 返回结果，如果没有找到则返回null
	}

	//判断永久版
	public static Boolean judgeSubscription(List<UserSubscriptionInfoVO> subscriptionInfoList,Integer platform){
		List<UserSubscriptionInfoVO> filteredList = subscriptionInfoList.stream()
				.filter(info -> !ObjectUtils.isEmpty(info.getPlatform())&& platform.equals(info.getPlatform())) // 排除AI记录
				.collect(Collectors.toList());
		if(filteredList.size()>0){
			return Boolean.TRUE;
		}
		return false;
	}
	private static int getLevelPriority(UserSubscriptionInfoVO info) {
		// 假设 levels 是以逗号分隔的字符串
		String[] levels = info.getLevels().split(",");
		int maxPriority = Integer.MIN_VALUE;

		for (String level : levels) {
			int lvl = Integer.parseInt(level.trim());
			maxPriority = Math.max(maxPriority, lvl);
		}

		// 定义优先级
		switch (maxPriority) {
			case 3: return 3; // 高级版
			case 2: return 2; // 标准版
			case 1: return 1; // 免费版
			default: return 0; // 未知或无效等级
		}
	}

	public static int getPlatforms(UserSubscriptionInfoVO info) {
		// 假设 platforms 是以逗号分隔的字符串
		String[] levels = info.getPlatforms().split(",");

		// 返回平台的数量（即数组的长度）
		return levels.length;
	}

	private static int getPlatformsCompare(UserSubscriptionInfoVO info) {
		// 假设 platforms 是以逗号分隔的字符串
		String levels = info.getPlatforms();

		// 将字符串按逗号分割成数组
		String[] platforms = levels.split(",");

		// 定义平台的优先级
		int priorityMac = 4;    // Mac > Windows > iOS > Android
		int priorityWindows = 3;
		int priorityIOS = 2;
		int priorityAndroid = 1;

		// 检查平台的优先级，并返回最优的平台的优先级值
		for (String platform : platforms) {
			switch (platform.trim().toLowerCase()) { // 使用 trim() 去除空格，toLowerCase() 处理大小写
				case "mac":
					return priorityMac;
				case "windows":
					return priorityWindows;
				case "ios":
					return priorityIOS;
				case "android":
					return priorityAndroid;
				default:
					break; // 如果不是这四个平台，忽略
			}
		}

		// 如果没有找到任何已知平台，返回默认值（如0）
		return 0;
	}

	//获取100年后的Date
	public static Date getDateByCycle(Date date, int cycle) {
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(date);
		calendar.add(Calendar.YEAR, cycle);
		return calendar.getTime();
	}

	public static String getUpperCaseSubstringFromUUID(int length) {
		// 生成 UUID
		String uuid = UUID.randomUUID().toString();

		// 获取从第 4 个字符开始的子字符串
		// 注意，Java 字符串是从索引 0 开始，因此要从索引 3 开始
		String substring = uuid.substring(3, Math.min(3 + length, uuid.length()));

		// 转换为大写
		return substring.toUpperCase();
	}

	// 临时保存图片到文件系统
	public static String saveTemporaryImage(MultipartFile image) {
		try {
			// 获取系统临时目录
			String tempDirectoryPath = System.getProperty("java.io.tmpdir") + "/temp_images/";
			File tempDirectory = new File(tempDirectoryPath);
			if (!tempDirectory.exists()) {
				tempDirectory.mkdirs();  // 如果目录不存在，则创建目录
			}

			// 获取文件的原始文件名
			String fileName = image.getOriginalFilename();
			String filePath = tempDirectoryPath + fileName;

			// 将图片保存到临时目录
			Path path = Paths.get(filePath);
			Files.write(path, image.getBytes());

			return filePath;  // 返回图片保存的路径
		} catch (IOException e) {
			e.printStackTrace();
			return null;
		}
	}

	// 删除临时文件
	public static void deleteTemporaryFile(String filePath) {
		File file = new File(filePath);
		if (file.exists()) {
			boolean deleted = file.delete();  // 删除文件
			if (deleted) {
				System.out.println("临时文件已删除: " + filePath);
			} else {
				System.out.println("删除临时文件失败: " + filePath);
			}
		}
	}
}
